#!/bin/bash

assert_name=$1

if [[ -z ${assert_name} ]]
then
    echo "usage: dw-deqo.bash <assert-name>"
    exit 1
fi

{
    dw get variable
    dw get assert ${assert_name}
} |

awk '

BEGIN { SUBSEP = ","; vars = 0; active_vars = 0; i = 0; }

$1 ~ /^bool:$/ { var_index[$2] = vars; var_name[vars] = $2; ++ vars }

$1 ~ /^active_vars.*/ {
    sub(/^.*=/,"");
    V[active_vars] = var_index[$0];
    ++ active_vars;
    states = 2 ** active_vars;
    value_column = 1 + active_vars;
}

$1 ~ /^0$|^1$/ {
    A[i] = $value_column;
    ++ i;
}

END {
    for (i=0; i<vars; ++i) printf("var[%d]=%s\n", i, var_name[i]);

    for (i=0; i<active_vars; ++i) printf("active_var[%d]=%s\n", i, var_name[V[i]]);

    states_to_penalize = 0;

    printf("state mask: ");
    for (i=0; i<states; ++i)
    {
        printf("%d", A[i]);
        states_to_penalize += (1 - A[i]);
        Aclone[i] = A[i];
    }
    printf("\n");

    printf("states_to_penalize = %d\n", states_to_penalize);

    term_limit = 3 ** active_vars;

    for (degree=1; degree<=active_vars; ++ degree)
    {
        best_term = -1;
        best_value = -1;

        printf("Consider terms of degree %d\n", degree);
        for (i=0; i<term_limit; ++i)
        {

            # Decode i into a "bit" string of length active_vars
            # except we are using digits 0, 1 and 2.  At the same
            # time, count the number of 0 and 1 digits in the string:

            i_copy = i;
            this_degree = 0;
            for (digit=0; digit<active_vars; ++ digit)
            {
                pos[digit] = (i_copy % 3);
                i_copy = (i_copy - pos[digit]) / 3;
                this_degree += (pos[digit] < 2);
            }

            # The number i now represents a term which has one of three
            # effects of each active variable.  If a "bit" location is
            # 0 or 1, then the corresponding active variable is forced
            # to be 0 or 1.  If a "bit" location is 2, then the corresponding
            # active variable is unconstrained.

            if (this_degree != degree) continue

            # We have found a term of the appropriate degree

            for (digit=active_vars-1; digit>=0; -- digit) printf("%d", pos[digit]);
            printf(" penalizes ");

            # Determine which states are penalized by this term:

            for (j=0; j<states; ++j)
            {
                j_copy = j;
                penalized[j] = 1;
		for (digit=0; digit<active_vars; ++ digit)
		{
		    bit = (j_copy % 2);
		    j_copy = (j_copy - bit) / 2;
                    if (pos[digit] == 2) continue;
                    if (bit != pos[digit]) penalized[j] = 0;
		}
                printf("%d", penalized[j]);
            }

            # Determine whether we can consider this term:

            printf(" ok ");
            ok = 1;
            for (j=0; j<states; ++j) if (A[j] == 1 && penalized[j] == 1) ok = 0;
            printf("%d", ok);

            if (ok) {
                # Compute the term value (number of unpenalized states penalized by this term):
                value = 0;
                for (j=0; j<states; ++j) value += (Aclone[j] == 0 && penalized[j] == 1);
                printf(" value=%d\n", value);

                if (value > best_value)
                {
                    best_value = value;
                    best_term = i;
                    for (j=0; j<states; ++j) best_penalized[j] = penalized[j];
                }
            } else {
                printf("\n");
            }
        }

        printf("best_value=%d best_term=%d\n", best_value, best_term);

        # If there is a useful term at this degree, include it and re-scan terms of this degree.
        # When there are no more states to penalize, exit.
        if (best_value > 0)
        {

            for (j=0; j<states; ++j)
            {
                if (Aclone[j] == 0 && best_penalized[j] == 1) -- states_to_penalize;
                if (best_penalized[j]) Aclone[j] = 1;
            }

            printf("term: ");
            i_copy = best_term;
            output_string = "";
            for (digit=0; digit<active_vars; ++ digit)
            {
                tridigit = (i_copy % 3);
                i_copy = (i_copy - tridigit) / 3;
                output_string = sprintf("%d",tridigit) output_string;
            }
            printf("%s\n", output_string);

	    printf("state mask: ");
	    for (j=0; j<states; ++j)
	    {
		printf("%d", Aclone[j]);
	    }
	    printf("\n");

            if (states_to_penalize == 0)
            {
                printf("done\n");
                exit;
            }

            -- degree;
        }
    }

}
'
