#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <string.h>

#include "qsage.h"

int *bool_state;

int hadamard_init()
{
  printf("Matrix size = %dx%d\n", N, N);

  bool_state = malloc(N * N * sizeof(int));
  memset(bool_state, 0, N * N * sizeof(int));

  return N * N;
}

#define ROW(i) (bool_state + (i)*N)

#define DOTPRODUCT(d,i,j) \
  { \
  int k, s = 0; \
  for (k=0; k<N; ++k) \
    s += ROW(i)[k] * ROW(j)[k]; \
  d = s; \
  }			   

double hadamard_obj(const int* state, size_t len)
{
  double obj = 0.0;
  int i, j;

  /* Convert 0/1 variables to -1/+1 variables: */
  for (i=0; i<N*N; ++i)
    bool_state[i] = 2*state[i] - 1;

  /* For each pair of rows, compute the dot product: */
  for (i=0; i<N; ++i)
    for (j=i+1; j<N; ++j)
      {
	int dp;
	DOTPRODUCT(dp,i,j);
	obj += (dp > 0) ? dp : -dp;
      }

  return obj;
}


void hadamard_final(const int* state, size_t len)
{
  int r, c;

  for (r=0; r<N; ++r)
    {
      for (c=0; c<N; ++c) printf("%2d ", ROW(r)[c]);

      printf("\n");
    }
}
